#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "plascolin.h"
#include "jpeg.h"

#include <sys/dir.h>	// directory content handling



/***********************************************
 *
 * Global variables, declared in plascolin.c (& main.c)
 *
 */
extern struct RGBImage img_r;		// right    image
extern struct RGBImage img_l;		// left     image
extern struct RGBImage img_a;		// anaglyph image

extern int x_offset;			// x_offset of the two stereo images
extern int y_offset;			// y_offset of the two stereo images


extern GtkWidget *Main_window;		/* -> declared in main.c     */
extern GtkWidget *Display2;		/* -> declared in main.c     */
extern GtkWidget *Display3;		/* -> declared in main.c     */

extern struct dirent **filelist;	// List of jpeg files in current workingdir
extern int imgnum;			// the number of leftimage in the list of jpeg files in current workingdir
extern unsigned int maxfiles;		// the maximum number of jpeg files in current workingdir
extern char leftfile[200];  	// the name of the left file
extern char rightfile[200];  	// the name of the right file

extern char workingdir[200];		/* name of working directory */

extern double disp_zoom;		/* Display zoom value        */
extern char disp_measure_modus;		/* Rangemodus 0=hf units     */
                                	/*            1=pov units    */
extern int disp_offset_x;		/* Display offset X value    */ 
extern int disp_offset_y;		/* Display offset Y value    */ 

extern char disp2_on;			/* Flag: wheather to show display2 window */
extern char disp3_on;			/* Flag: wheather to show display3 window */
extern char disp2_hmirror_on;		/* Flag: wheather to mirror display2*/
extern char disp3_hmirror_on;		/* Flag: wheather to mirror display3 */




/***********************************************
 *
 * Main Window
 *
 */
void
on_Main_window_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
 write_config_file();		// write configuration data before exiting
 gtk_main_quit();
}


void
on_Main_window_check_resize            (GtkContainer    *container,
                                        gpointer         user_data)
{

}


/******************
 *   Keyboard action on main window  
 */
gboolean
on_Main_window_key_press_event         (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  printf("KEY PRESSED: %i\n",event->keyval);
  if(event->keyval==43 || event->keyval==65451)	  zoom_in();	// zoom in
  if(event->keyval==45 || event->keyval==65453)	  zoom_out(); 	// zoom out
  if(event->keyval==122)		// 'z' zoom to fit
  {
	zoomtofit();
  }
  if(event->keyval==65363)		// 'right'
  { 
    x_offset=x_offset - 3 ;
    create_anaglyph();
  }
  if(event->keyval==65361)		// 'left'  
  { 
    x_offset=x_offset + 3 ;
    create_anaglyph();
  }
  if(event->keyval==65362)		// 'up'
  { 
    y_offset=y_offset + 3 ;
    create_anaglyph();
  }
  if(event->keyval==65364)		// 'down'
  { 
    y_offset=y_offset - 3 ;
    create_anaglyph();
  }
  if(event->keyval==102) 	// 'f' switch to fillscreen
  {
    gtk_widget_hide(lookup_widget(GTK_WIDGET(Main_window),"hbox2"));
    gtk_widget_hide(lookup_widget(GTK_WIDGET(Main_window),"vbox2"));
    gtk_widget_hide(lookup_widget(GTK_WIDGET(Main_window),"vscrollbar1"));
    gtk_widget_hide(lookup_widget(GTK_WIDGET(Main_window),"hscrollbar1"));
    gtk_widget_hide(lookup_widget(GTK_WIDGET(Main_window),"hbox1"));
    gtk_window_fullscreen(Main_window);
  }
  if(event->keyval==65307)	// 'esc' switch back to indow mode
  {
    gtk_widget_show(lookup_widget(GTK_WIDGET(Main_window),"hbox2"));
    gtk_widget_show(lookup_widget(GTK_WIDGET(Main_window),"vbox2"));
    gtk_widget_show(lookup_widget(GTK_WIDGET(Main_window),"vscrollbar1"));
    gtk_widget_show(lookup_widget(GTK_WIDGET(Main_window),"hscrollbar1"));
    gtk_widget_show(lookup_widget(GTK_WIDGET(Main_window),"hbox1"));
    gtk_window_unfullscreen(Main_window);
  }
  if(event->keyval==116)		// 't' (test)
  {
	test();
  }

  if(event->keyval==115)		// 's' save JPEG file
  { 
	write_file();
  }

  if(event->keyval==65366)  next_pair();
  if(event->keyval==65365)  prev_pair();
 
  return FALSE;
}







//******************************************************************************************





void
on_button2_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
 open_files();
}


void
on_button3_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
 write_file();
}


void
on_button4_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
 exit_appl();
}




/***********************************************************************
 *
 *   Handle the scrollbars (horizontal & vertical)
 *
 *   If imagebuffer is larger than display (drawable) we are able to
 *   change the scrollbar-position (otherwise we are not able to do so).
 *   In this case we need to set the offset of the display and redraw.
 * 
 */
void
on_hscrollbar1_value_changed           (GtkRange        *range,
                                        gpointer         user_data)
{
  GtkRange *hscrollbar = (GtkRange*)lookup_widget(GTK_WIDGET(Main_window),"hscrollbar1");
  GtkAdjustment *hadjust;

  // get scrollbar values
  hadjust = gtk_range_get_adjustment(hscrollbar);
  disp_offset_x=hadjust->value;
  display();
}

void
on_vscrollbar1_value_changed           (GtkRange        *range,
                                        gpointer         user_data)
{
  GtkRange *vscrollbar = (GtkRange*)lookup_widget(GTK_WIDGET(Main_window),"vscrollbar1");
  GtkAdjustment *vadjust;

  // get scrollbar values
  vadjust = gtk_range_get_adjustment(vscrollbar);
  disp_offset_y=vadjust->value;
  display();
}










/***********************************************
 *
 * Drawing Area Events
 *
 */
gboolean
on_drawingarea1_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_drawingarea1_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  display();
}


gboolean
on_drawingarea1_configure_event        (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{

  return FALSE;
}



/******************
 *
 *   Z O O M I N G  
 *
 *
 */

void
on_bt_zoom_in_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
 zoom_in();
}


void
on_bt_zoom_out_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
 zoom_out();
}


void
on_bt_zoom_fit_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
 zoomtofit();
}



/***********************************************
 *
 * Next & Previous Image Pair
 *
 */
void
on_bt_nextpair_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
 next_pair();
}


void
on_bt_prevpair_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
 prev_pair();
}
















/******************************************************
 *  Function:  static int isjpg (struct dirent *d);
 *
 *  Description: 
 *  - This function returns 1 in case file in dirent *d is a jpeg file.
 *  - if not return 0 
 *  - used for scandir(workingdir, &filelist, isjpg, alphasort);
 *
*/
static int
isjpg (struct dirent *d)
{
  char* p;
  if ((p = strrchr(d->d_name, '.')) == NULL)
   return 0;
  if (strcmp(p, ".jpg") == 0)
   return 1;
  if (strcmp(p, ".jpeg") == 0)
   return 1;
  if (strcmp(p, ".JPG") == 0)
   return 1;
  if (strcmp(p, ".JPEG") == 0)
   return 1;
  else
   return 0;
}


/***********************************************
 *
 * FileOpen Dialog
 *
 */
void
on_bt_FileOpen_Cancel_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
 GtkWidget * openfile = lookup_widget(GTK_WIDGET(button),"FileOpen");
 gtk_widget_destroy(openfile);
}


void
on_bt_FileOpen_Open_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
 GtkWidget *openfile = lookup_widget(GTK_WIDGET(button),"FileOpen");
 gchar *selected_filename;
 char *tmp;  		// a temporary pointer
 char tmp2[200];	// a temporary string
 char tmp3[200];	// a temporary string

 
 unsigned int x;
 int n;

 selected_filename = gtk_file_chooser_get_filename (GTK_FILE_SELECTION(openfile));
 gtk_widget_destroy(openfile);


 // get workingdir 
 sprintf(workingdir,"%s",selected_filename); 
 tmp = strrchr(workingdir,47) ;  // prepare workingdir 
 *tmp=0;

 // get name of leftfile 
 sprintf(leftfile,"%s",strrchr(selected_filename,47)+1);
 
 // generate a file list & find out which file in the list is the selected one
 // file list
 n = scandir(workingdir, &filelist, isjpg, alphasort);
 //printf("Number of matching files: %i\n\n",n);
 maxfiles = n;
 if (n < 0)  printf("scandir() has failed\n");
 else
   for(x=0; x<n; x++)
   {
     if(strcmp(leftfile,filelist[x]->d_name)==0)
     {
      imgnum = x;
      strcpy(rightfile,filelist[x+1]->d_name);
     }
   }

 printf("leftfile:\t%s\n",leftfile);		
 printf("rightfile:\t%s\n",rightfile);

 // convert leftfile & rightfile back to absolute path names
 sprintf(tmp2,"%s/%s",workingdir,leftfile);
 if(img_l.data!=0)   free(img_l.data);	// free buffer if it was used before
 img_l.data= read_JPEG_file(tmp2,  &img_l.width, &img_l.height);

 sprintf(tmp2,"%s/%s",workingdir,rightfile);
 if(img_r.data!=0)   free(img_r.data);	// free buffer if it was used before
 img_r.data= read_JPEG_file(tmp2, &img_r.width, &img_r.height);

 // display file names and size information in statusbar labels
 sprintf(tmp3,"%s  %ix%i",leftfile, img_l.width,img_l.height);
 gtk_label_set_text (lookup_widget(GTK_WIDGET(Main_window),"lb_leftfilename"), tmp3 );
 sprintf(tmp3,"%s  %ix%i",rightfile,img_r.width,img_r.height);
 gtk_label_set_text (lookup_widget(GTK_WIDGET(Main_window),"lb_rightfilename"), tmp3 ); 

 create_anaglyph();
 if(disp2_on==1)   display2();
 if(disp3_on==1)   display3();
 zoomtofit();
}







//-----------------------------------------------
// Create additional display windows for right and left view
void
on_button9_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  if(disp2_on==0)
  {
   Display2 = create_Display2();
   gtk_widget_show (Display2);
   disp2_on = 1 ;
  }
}


void
on_button10_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  if(disp3_on==0)
  {
   Display3 = create_Display3();
   gtk_widget_show (Display3);
   disp3_on = 1 ;
  }
}



//-----------------------------------------------
// Close-Buttons in Display2 & Display3 Windows

void
on_button6_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
 gtk_widget_destroy (Display2);	
 disp2_on = 0 ;
}


void
on_button7_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
 gtk_widget_destroy (Display3);
 disp3_on = 0 ;
}





//-----------------------------------------------
// Fullscreen buttons in Display2 & 3 Windows

void
on_button12_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(lookup_widget(GTK_WIDGET(Display2),"hbox7"));
  gtk_window_fullscreen(Display2);
}


void
on_button13_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(lookup_widget(GTK_WIDGET(Display3),"hbox8"));
  gtk_window_fullscreen(Display3);
}



//-----------------------------------------------
// H Mirror Buttons in Display2 & 3
void
on_button14_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
 if(disp2_hmirror_on==0)
  disp2_hmirror_on=1;
 else
  disp2_hmirror_on=0;
 display2();
}

void
on_button15_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
 if(disp3_hmirror_on==0)
  disp3_hmirror_on=1;
 else
  disp3_hmirror_on=0;
 display3();
}




//-----------------------------------------------------------------
// Display2

gboolean
on_Display2_destroy_event              (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

  return FALSE;
}


void
on_Display2_check_resize               (GtkContainer    *container,
                                        gpointer         user_data)
{
//  if(disp2_on==1)   display2();
}


gboolean
on_Display2_key_press_event            (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  printf("KEY PRESSED: %i\n",event->keyval);
  if(event->keyval==65307)	// 'esc' switch back to window mode
  {
    gtk_widget_show(lookup_widget(GTK_WIDGET(Display2),"hbox7"));
    gtk_window_unfullscreen(Display2);
  }
  if(event->keyval==65366)  next_pair();
  if(event->keyval==65365)  prev_pair();
  return FALSE;
}


gboolean
on_drawingarea2_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  display2();
  return FALSE;
}



//-----------------------------------------------------------------
// Display3

gboolean
on_Display3_destroy_event              (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

  return FALSE;
}


void
on_Display3_check_resize               (GtkContainer    *container,
                                        gpointer         user_data)
{
//  if(disp3_on==1)   display3();
}


gboolean
on_Display3_key_press_event            (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  printf("KEY PRESSED: %i\n",event->keyval);
  if(event->keyval==65307)	// 'esc' switch back to window mode
  {
    gtk_widget_show(lookup_widget(GTK_WIDGET(Display3),"hbox8"));
    gtk_window_unfullscreen(Display3);
  }
  if(event->keyval==65366)  next_pair();
  if(event->keyval==65365)  prev_pair();
  return FALSE;
}



gboolean
on_drawingarea3_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  display3();
  return FALSE;
}



//-----------------------------------------------
// Testbutton
void
on_button8_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  if(disp2_on==1)   display2();
  if(disp3_on==1)   display3();
  //gtk_window_unfullscreen(Display2);
}






